@@ -4,44 +4,39 @@ import allure
 import uiautomator2 as u2
 from qrunner.utils.config import conf
 from qrunner.utils.log import logger
-from baseImage import Image, Rect
+from qrunner.utils.exceptions import DeviceNotFoundException, ScreenFailException
+from qrunner.core.android.common import get_device_list
 
 
 class AndroidDriver(object):
-    # _instance = {}
-    #
-    # def __new__(cls, serial_no=None):
-    #     if not serial_no:
-    #         serial_no = conf.get_name('app', 'serial_no')
-    #     if serial_no not in cls._instance:
-    #         cls._instance[serial_no] = super().__new__(cls)
-    #     return cls._instance[serial_no]
+    _instance = {}
 
-    def __init__(self, serial_no=None, pkg_name=None):
+    def __new__(cls, serial_no=None):
+        if not serial_no:
+            serial_no = conf.get_item('android', 'serial_no')
+        if serial_no not in cls._instance:
+            cls._instance[serial_no] = super().__new__(cls)
+        return cls._instance[serial_no]
+
+    def __init__(self, serial_no=None):
         if serial_no is None:
-            serial_no = conf.get_item('app', 'serial_no')
-        self.serial_no = serial_no
-        if pkg_name is None:
-            pkg_name = conf.get_item('app', 'pkg_name')
-        self.pkg_name = pkg_name
+            serial_no = conf.get_item('android', 'serial_no')
+        if serial_no in get_device_list():
+            self.serial_no = serial_no
+        else:
+            raise DeviceNotFoundException(msg=f'设备 {serial_no} 未连接')
+        self.pkg_name = conf.get_item('android', 'pkg_name')
 
         logger.info(f'启动 android driver for {self.serial_no}')
         self.d = u2.connect(self.serial_no)
-        self.session = None
 
-    # @classmethod
-    # def get_instance(cls, serial_no=None):
-    #     """Create singleton"""
-    #     if serial_no not in cls._instance:
-    #         logger.info(f'[{serial_no}] Create android driver singleton')
-    #         return AndroidDriver(serial_no)
-    #     return AndroidDriver._instance[serial_no]
-
-    # @classmethod
-    # def get_remote_instance(cls, server_url, token):
-    #     device = Device(server_url, token)
-    #     d = u2.connect(device.get_device())
-    #     return d, device
+    @classmethod
+    def get_instance(cls, serial_no=None):
+        """Create singleton"""
+        if serial_no not in cls._instance:
+            logger.info(f'[{serial_no}] Create android driver singleton')
+            return AndroidDriver(serial_no)
+        return AndroidDriver._instance[serial_no]
 
     def uninstall_app(self, pkg_name=None):
         if not pkg_name:
@@ -53,9 +48,11 @@ class AndroidDriver(object):
         logger.info(f'覆盖安装应用: {apk_path}')
         self.d.app_install(apk_path)
 
-    def new_install_app(self, apk_path):
+    def new_install_app(self, apk_path, pkg_name=None):
         logger.info(f'先卸载再安装应用: {apk_path}')
-        self.uninstall_app(self.pkg_name)
+        if not pkg_name:
+            pkg_name = self.pkg_name
+        self.uninstall_app(pkg_name)
         self.d.app_install(apk_path)
 
     def start_app(self, pkg_name=None):
@@ -223,49 +220,35 @@ class AndroidDriver(object):
         logger.info(is_running)
         return is_running
 
-    def screenshot(self, file_name, rect=None):
+    def screenshot(self, file_name):
         """
-        截图并保存到预定路径并上传到allure报告
+        截图并保存到预定路径
         @param file_name: foo.png or fool
-        @param rect: 需要标记控件的左上角坐标和宽高
         @return:
         """
         # 把文件名处理成test.png的样式
-        if '.' in file_name:
-            file_name = file_name.split(r'.')[0]
-        # 截图并保存到当前目录的images文件夹中
-        img_dir = os.path.join(os.getcwd(), 'images')
-        if os.path.exists(img_dir) is False:
-            os.mkdir(img_dir)
-        time_str = time.strftime('%Y年%m月%d日 %H时%M分%S秒')
-        file_path = os.path.join(img_dir, f'{time_str}-{file_name}.png')
-        logger.info(f'截图保存至: {file_path}')
-        self.d.screenshot(file_path)
+        try:
+            if '.' in file_name:
+                file_name = file_name.split(r'.')[0]
+            # 截图并保存到当前目录的images文件夹中
+            img_dir = os.path.join(os.getcwd(), 'images')
+            if os.path.exists(img_dir) is False:
+                os.mkdir(img_dir)
+            time_str = time.strftime('%Y年%m月%d日 %H时%M分%S秒')
+            file_path = os.path.join(img_dir, f'{time_str}-{file_name}.png')
+            self.d.screenshot(file_path)
+            # 上传allure报告
+            allure.attach.file(file_path, attachment_type=allure.attachment_type.PNG, name=f'{file_name}.png')
+            return file_path
+        except Exception as e:
+            raise ScreenFailException(f'{file_name} 截图失败\n{str(e)}')
 
-        # 给图片打上标记
-        if rect is not None:
-            img = Image(file_path)
-            img.rectangle(Rect(*rect), color=(0, 0, 255), thickness=3)
-            img.imwrite(file_path)
-
-        # 截图并上传到allure报告
-        logger.info(f'截图上传至allure报告')
-        allure.attach.file(file_path,
-                           attachment_type=allure.attachment_type.PNG,
-                           name=f'{time_str}-{file_name}.png')
-        return file_path
-
-    # def upload_pic(self, filename):
-    #     self.screenshot('tmp.png')
-    #     allure.attach.file(
-    #         'tmp.png',
-    #         attachment_type=allure.attachment_type.PNG,
-    #         name=f'{filename}-{get_time()}'
-    #     )
-    #     os.remove('tmp.png')
-
-    # def input_password(self, text):
-    #     self.d(focused=True).set_text(text)
+    def upload_pic(self, file_name):
+        # 截图并保存
+        file_path = self.screenshot(file_name)
+        # 上传allure报告
+        allure.attach.file(file_path, attachment_type=allure.attachment_type.PNG, name=f'{file_name}.png')
+        logger.debug(f'[截图并上传报告] {file_path}')
 
     def get_page_content(self):
         return self.d.dump_hierarchy()
@@ -311,7 +294,7 @@ class AndroidDriver(object):
         @return:
         """
         logger.info(f'点击key: {key}')
-        self.press(key)
+        self.d.press(key)
 
     def click(self, x, y):
         logger.info(f'单击坐标: {x},{y}')
@@ -321,6 +304,8 @@ class AndroidDriver(object):
         logger.info(f'批量点击弹窗: {alert_list}')
         with self.d.watch_context() as ctx:
             for alert in alert_list:
+                if 'id/' in alert:
+                    alert = f'{conf.get_item("android", "pkg_name")}:{alert}'
                 ctx.when(alert).click()
             ctx.wait_stable()
 
