@@ -1,95 +1,66 @@
-import binascii
 import os
+import shutil
 import subprocess
 import time
-import allure
-import requests
-import wda
-from baseImage import Image, Rect
 
+import allure
+import wda
+
+from qrunner.core.ios.common import get_device_list, get_tcp_port
 from qrunner.utils.log import logger
 from qrunner.utils.config import conf
+from qrunner.utils.exceptions import DeviceNotFoundException, ScreenFailException
 
-# # 定义最大重连次数
-# max_retry = 3
-#
-#
-# def launch_wda(serial_no):
-#     logger.info(f'{serial_no} 开始启动wda服务')
-#     cmd = 'tidevice -u {0} xctest'.format(serial_no)
-#     subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
-#
-#
-# # 杀掉tidevice进程
-# def stop_wda():
-#     logger.info('\n停止wda服务')
-#     try:
-#         if os.name == 'nt':
-#             os.popen('taskkill /f /im tidevice.exe')
-#         else:
-#             os.popen("ps -ef | grep tidevice | awk '{print $2}' | xargs kill -9")
-#     except Exception as e:
-#         logger.error(f'停止wda服务失败: {str(e)}')
-#     else:
-#         logger.info('停止wda服务成功')
-#
-#
-# def relaunch_wda(func):
-#     def wrapper(self, *args, **kwargs):
-#         return _inner_retry(self, func, *args, **kwargs)
-#     return wrapper
-#
-#
-# def _inner_retry(self, func, *args, **kwargs):
-#     global max_retry
-#
-#     while max_retry > 0:
-#         try:
-#             return func(self, *args, **kwargs)
-#         except (requests.exceptions.ConnectionError, wda.WDAError,
-#                 binascii.Error) as _:
-#             time.sleep(3)
-#             logger.info(f'开始重连wda')
-#             # serial_no = conf.get_name('app', 'serial_no')
-#             launch_wda(self.serial_no)
-#             max_retry -= 1
-#             logger.info(f'剩余重连次数: {max_retry}')
-#     return func(self, *args, **kwargs)
+
+def _start_wda_xctest(udid: str, port, wda_bundle_id=None) -> bool:
+    xctool_path = shutil.which("tidevice")
+    logger.info(f"WDA is not running, exec: {xctool_path} -u {udid} wdaproxy --port {port} -B {wda_bundle_id}")
+    args = []
+    if udid:
+        args.extend(['-u', udid])
+    args.append('wdaproxy')
+    args.extend(['--port', str(port)])
+    if wda_bundle_id is not None:
+        args.extend(['-B', wda_bundle_id])
+    p = subprocess.Popen([xctool_path] + args)
+    time.sleep(3)
+    if p.poll() is not None:
+        logger.warning("xctest launch failed")
+        return False
+    return True
 
 
 class IosDriver(object):
-    # _instance = {}
-    #
-    # def __new__(cls, serial_no=None, bundle_id=None):
-    #     if not serial_no:
-    #         serial_no = conf.get_name('app', 'serial_no')
-    #     if serial_no not in cls._instance:
-    #         cls._instance[serial_no] = super().__new__(cls)
-    #     return cls._instance[serial_no]
+    _instance = {}
 
-    def __init__(self, serial_no=None, pkg_name=None):
+    def __new__(cls, serial_no=None, bundle_id=None):
         if not serial_no:
-            serial_no = conf.get_item('app', 'serial_no')
-        self.serial_no = serial_no
-        if not pkg_name:
-            pkg_name = conf.get_item('app', 'pkg_name')
-        self.bundle_id = pkg_name
-        logger.info(self.bundle_id)
-        logger.info(f'启动 ios driver for {self.serial_no}')
-        self.d = wda.USBClient(self.serial_no, port=8100)
+            serial_no = conf.get_item('ios', 'serial_no')
+        if serial_no not in cls._instance:
+            cls._instance[serial_no] = super().__new__(cls)
+        return cls._instance[serial_no]
 
-    # @classmethod
-    # def get_instance(cls, serial_no=None):
-    #     if serial_no not in cls._instance:
-    #         logger.info(f'{serial_no} Create ios driver singleton')
-    #         return IosDriver(serial_no)
-    #     return IosDriver._instance[serial_no]
-    #
-    # @classmethod
-    # def get_remote_instance(cls, server_url, token):
-    #     device = Device(server_url, token)
-    #     d = wda.Client(device.get_device(platform='apple'))
-    #     return d, device
+    def __init__(self, serial_no=None):
+        self.serial_no = serial_no if serial_no else conf.get_item('ios', 'serial_no')
+        if serial_no in get_device_list():
+            self.serial_no = serial_no
+        else:
+            raise DeviceNotFoundException(msg=f'设备 {serial_no} 未连接')
+        self.bundle_id = conf.get_item('ios', 'pkg_name')
+        logger.info(f'启动 ios driver for {self.serial_no}')
+        # 启动wda
+        port = get_tcp_port(self.serial_no)
+        self.d = wda.Client(f"http://localhost:{port}")
+        if not self.d.is_ready():
+            logger.info('wda未启动，开始启动wda')
+            _start_wda_xctest(self.serial_no, port=port)
+
+    @classmethod
+    def get_instance(cls, serial_no=None):
+        if serial_no not in cls._instance:
+            logger.info(f'{serial_no} Create ios driver singleton')
+            return IosDriver(serial_no)
+        return IosDriver._instance[serial_no]
 
     def install_app(self, ipa_url):
         cmd = f'tidevice -u {self.serial_no} install {ipa_url}'
@@ -101,8 +72,9 @@ class IosDriver(object):
         else:
             logger.info(f'{self.serial_no} 安装应用{ipa_url}失败，因为{output}')
 
-    def new_install_app(self, ipa_url):
-        self.uninstall_app(self.bundle_id)
+    def new_install_app(self, ipa_url, bundle_id=None):
+        pkg_name = bundle_id if bundle_id else self.bundle_id
+        self.uninstall_app(pkg_name)
         cmd = f'tidevice -u {self.serial_no} install {ipa_url}'
         logger.info(f'安装应用: {ipa_url}')
         output = subprocess.getoutput(cmd)
@@ -113,16 +85,15 @@ class IosDriver(object):
             logger.info(f'{self.serial_no} 安装应用{ipa_url}失败，因为{output}')
 
     def uninstall_app(self, bundle_id=None):
-        if not bundle_id:
-            bundle_id = self.bundle_id
-        cmd = f'tidevice -u {self.serial_no} uninstall {bundle_id}'
-        logger.info(f'卸载应用: {bundle_id}')
+        pkg_name = bundle_id if bundle_id else self.bundle_id
+        cmd = f'tidevice -u {self.serial_no} uninstall {pkg_name}'
+        logger.info(f'卸载应用: {pkg_name}')
         output = subprocess.getoutput(cmd)
         if 'Complete' in output.split()[-1]:
-            logger.info(f'{self.serial_no} 卸载应用{bundle_id} 成功')
+            logger.info(f'{self.serial_no} 卸载应用{pkg_name} 成功')
             return
         else:
-            logger.info(f'{self.serial_no} 卸载应用{bundle_id}失败，因为{output}')
+            logger.info(f'{self.serial_no} 卸载应用{pkg_name}失败，因为{output}')
 
     def start_app(self, bundle_id=None):
         if not bundle_id:
@@ -167,45 +138,52 @@ class IosDriver(object):
         logger.info(f'输入: {value}')
         self.d.send_keys(value)
 
-    def screenshot(self, file_name, rect=None):
+    def screenshot(self, file_name):
         """
-        截图并保存到预定路径并上传到allure报告
+        截图并保存到预定路径
         @param file_name: foo.png or fool
-        @param rect: 需要标记控件左上角坐标以及宽高
         @return:
         """
         # 把文件名处理成test.png的样式
-        if '.' in file_name:
-            file_name = file_name.split(r'.')[0]
-        # 截图并保存到当前目录的images文件夹中
-        img_dir = os.path.join(os.getcwd(), 'images')
-        if os.path.exists(img_dir) is False:
-            os.mkdir(img_dir)
-        time_str = time.strftime('%Y年%m月%d日 %H时%M分%S秒')
-        file_path = os.path.join(img_dir,
-                                 f'{time_str}-{file_name}.png')
-        logger.info(f'截图保存至: {file_path}')
-        self.d.screenshot(file_path)
+        try:
+            if '.' in file_name:
+                file_name = file_name.split(r'.')[0]
+            # 截图并保存到当前目录的images文件夹中
+            img_dir = os.path.join(os.getcwd(), 'images')
+            if os.path.exists(img_dir) is False:
+                os.mkdir(img_dir)
+            time_str = time.strftime('%Y年%m月%d日 %H时%M分%S秒')
+            file_path = os.path.join(img_dir, f'{time_str}-{file_name}.png')
+            self.d.screenshot(file_path)
+            # 上传allure报告
+            allure.attach.file(file_path, attachment_type=allure.attachment_type.PNG, name=f'{file_name}.png')
+            return file_path
+        except Exception as e:
+            raise ScreenFailException(f'{file_name} 截图失败\n{str(e)}')
 
-        # 标记
-        if rect is not None:
-            img = Image(file_path)
-            img.rectangle(Rect(*rect), color=(0, 0, 255), thickness=3)
-            img.imwrite(file_path)
-
-        # 截图并上传到allure报告
-        logger.info(f'截图上传至allure报告')
-        allure.attach.file(file_path,
-                           attachment_type=allure.attachment_type.PNG,
-                           name=f'{time_str}-{file_name}')
-        return file_path
-
-    # def upload_pic(self, filename):
-    #     self.screenshot('tmp.png')
-    #     allure.attach.file('tmp.png',
-    #                        attachment_type=allure.attachment_type.PNG,
-    #                        name=f'{filename}-{get_time()}')
-    #     os.remove('tmp.png')
+    # def screenshot_and_mark(self, file_name, rect):
+    #     """给图片指定范围画上红框
+    #     rect: [x, y, width, height]
+    #     x: 左上坐标x
+    #     y：左上角坐标y
+    #     width：矩形宽度
+    #     height：矩形高度
+    #     """
+    #     # 把文件名处理成test.png的样式
+    #     if '.' in file_name:
+    #         file_name = file_name.split(r'.')[0]
+    #     # 截图并保存到当前目录的images文件夹中
+    #     img_dir = os.path.join(os.getcwd(), 'images')
+    #     if os.path.exists(img_dir) is False:
+    #         os.mkdir(img_dir)
+    #     time_str = time.strftime('%Y年%m月%d日 %H时%M分%S秒')
+    #     file_path = os.path.join(img_dir, f'{time_str}-{file_name}.png')
+    #     self.d.screenshot(file_path)
+    #     # 画框
+    #     ImageRecognition.mark(file_path, rect)
+    #     # 上传allure报告
+    #     allure.attach.file(file_path, attachment_type=allure.attachment_type.PNG, name=f'{file_name}.png')
+    #     return file_path
 
     @property
     def page_content(self):
