@@ -1,33 +1,61 @@
 import inspect
-import threading
 from typing import Union
 from wda import Selector
 from qrunner.utils.log import logger
 from qrunner.utils.config import conf
 from qrunner.core.ios.driver import IosDriver
-from multiprocessing import Pool
-from selenium.common.exceptions import NoSuchElementException
+from qrunner.utils.exceptions import NoSuchElementException, ElementNameEmptyException
 
 
 class IosElement(object):
     """
     IOS原生元素定义
     """
-    def __init__(self, driver=None, *args, **kwargs):
-        # 从kwargs中删掉并返回index
-        self._index = kwargs.pop('index', 0)
-        self.desc = kwargs.pop('desc', '')
-        # 参数初始化
-        self._xpath = kwargs.get('xpath', '')
+    def __init__(self,
+                 name=None,
+                 label=None,
+                 value=None,
+                 text=None,
+                 cname=None,
+                 xpath=None,
+                 index=None,
+                 desc=None):
+        """
+        param name,
+        param label,
+        param value,
+        param text,
+        param cname：className,
+        param xpath,
+        param index: 索引,
+        param desc: 控件名称
+        """
+        # 参数处理
+        kwargs = {}
+        if name is not None:
+            kwargs['name'] = name
+        if label is not None:
+            kwargs['label'] = label
+        if value is not None:
+            kwargs['value'] = value
+        if text is not None:
+            kwargs['text'] = text
+        if cname is not None:
+            kwargs['className'] = cname
+        self._xpath = xpath
+        self._index = index if index is not None else 0
+        self.desc = desc
+        if self.desc is None:
+            raise ElementNameEmptyException('请设置控件名称')
         self._kwargs = kwargs
-        self._element: Union[Selector]
-        self._driver: IosDriver = driver
-        self._serial_no = self._driver.serial_no
-        self._pkg_name = self._driver.bundle_id
+
+        # 驱动初始化
+        _serial_no = conf.get_item('ios', 'serial_no')
+        self._driver = IosDriver.get_instance(_serial_no)
         self._d = self._driver.d
 
     # 多线程处理异常弹窗
-    def handle_alert(self):
+    def handle_alert(self, alert_list):
         """
         根据不同定位方式进行点击
         @return:
@@ -39,48 +67,89 @@ class IosElement(object):
                 element = self._d(label=loc)
             element.click_exists()
 
-        # 多线程执行点击过程
-        alert_list = conf.get_item('app', 'ios_alert')
-        print(f'异常弹窗列表: {alert_list}')
-        for alert in alert_list:
-            click_alert(alert)
+        # 多线程存在问题，目前没有使用多线程
+        if alert_list:
+            print(f'处理弹窗: {alert_list}')
+            for alert in alert_list:
+                click_alert(alert)
 
-    def _find_element(self, retry=3, timeout=3):
+    # def _find_element(self, retry=3, timeout=3):
+    #     """
+    #     循环查找元素，查找失败先处理弹窗后重试
+    #     @param retry: 重试次数
+    #     @param timeout: 单次查找超时时间
+    #     @return:
+    #     """
+    #     self._element = self._d.xpath(self._xpath) if \
+    #         self._xpath else self._d(**self._kwargs)[self._index]
+    #     self.handle_alert()
+    #     while not self._element.wait(timeout=timeout):
+    #         if retry > 0:
+    #             retry -= 1
+    #             logger.warning(f'重试 查找元素： {self._kwargs}')
+    #             self.handle_alert()
+    #         else:
+    #             frame = inspect.currentframe().f_back
+    #             caller = inspect.getframeinfo(frame)
+    #             logger.warning(f'【{caller.function}:{caller.lineno}】未找到元素 {self._kwargs}')
+    #             return None
+    #     return self._element
+
+    def _find_element(self, retry=3, timeout=3, alert_list=None):
         """
         循环查找元素，查找失败先处理弹窗后重试
         @param retry: 重试次数
         @param timeout: 单次查找超时时间
         @return:
         """
-        self._element = self._d.xpath(self._xpath) if \
+        element = self._d.xpath(self._xpath) if \
             self._xpath else self._d(**self._kwargs)[self._index]
-        self.handle_alert()
-        while not self._element.wait(timeout=timeout):
-            if retry > 0:
-                retry -= 1
-                logger.warning(f'重试 查找元素： {self._kwargs}')
-                self.handle_alert()
+        self.handle_alert(alert_list)
+        cur_retry = retry
+        while not element.wait(timeout=timeout):
+            if cur_retry > 0:
+                logger.warning(f'第{retry-cur_retry+1}次重试，查找元素： {self._kwargs},{self._index}')
+                cur_retry -= 1
+                self.handle_alert(alert_list)
             else:
                 frame = inspect.currentframe().f_back
                 caller = inspect.getframeinfo(frame)
                 logger.warning(f'【{caller.function}:{caller.lineno}】未找到元素 {self._kwargs}')
                 return None
-        return self._element
+        elements = self._d.xpath(self._xpath).find_elements() if \
+            self._xpath else self._d(**self._kwargs)
+        return elements
 
-    def get_element(self, retry=3, timeout=3):
+    def get_elements(self, retry=3, timeout=3, alert_list=None):
         """
         针对元素定位失败的情况，抛出NoSuchElementException异常
         @param retry:
         @param timeout:
+        @param alert_list
         @return:
         """
-        element: Union[Selector] = self._find_element(retry=retry, timeout=timeout)
-        if element is None:
+        elements: Union[Selector] = self._find_element(retry=retry, timeout=timeout, alert_list=alert_list)
+        if elements is None:
             self._driver.screenshot(f'[控件 {self.desc}] 定位失败')
             raise NoSuchElementException(f'[控件 {self.desc}] 定位失败')
         else:
-            logger.debug('定位成功')
-        return element
+            self._driver.screenshot(self.desc)
+        return elements
+
+    # def __getitem__(self, index):
+    #     elements = self.get_elements()
+    #     return elements[index]
+
+    def get_element(self, retry=3, timeout=3, alert_list=None):
+        """
+        针对元素定位失败的情况，抛出NoSuchElementException异常
+        @param retry:
+        @param timeout:
+        @param alert_list
+        @return:
+        """
+        elements = self.get_elements(retry=retry, timeout=timeout, alert_list=alert_list)
+        return elements[self._index]
 
     @property
     def info(self):
@@ -156,13 +225,22 @@ class IosElement(object):
         logger.info(flag)
         return flag
 
-    def click(self):
+    def click(self, retry=3, timeout=3, alert_list=None):
+        """
+        @param: retry，重试次数
+        @param: timeout，每次重试超时时间
+        @param: alert_list，异常弹窗列表
+        """
         logger.info(f'点击元素: {self._kwargs},{self._index}')
-        self.get_element().click()
+        self.get_element(retry=retry, timeout=timeout, alert_list=alert_list).click()
 
-    def click_exists(self):
+    # def click_no_retry(self):
+    #     logger.info(f'点击元素: {self._kwargs},{self._index}')
+    #     self.get_element(retry=0, timeout=1).click()
+
+    def click_exists(self, timeout=3):
         logger.info(f'存在才点击元素: {self._kwargs},{self._index}')
-        if self.exists():
+        if self.exists(timeout=timeout):
             self.click()
 
     def clear_text(self):
@@ -197,6 +275,7 @@ class IosElement(object):
         self.get_element().swipe("down")
 
     def child(self, *args, **kwargs):
+        """不能用于PageObject，会导致在应用启动前进行元素识别"""
         return self.get_element().child(*args, **kwargs)
 
 
