@@ -1,5 +1,4 @@
 import inspect
-import time
 import platform
 from selenium.webdriver.common.by import By
 from selenium.webdriver.common.keys import Keys
@@ -7,9 +6,10 @@ from selenium.webdriver.common.action_chains import ActionChains
 from selenium.webdriver.support import expected_conditions as EC
 from selenium.webdriver.support.select import Select
 from selenium.webdriver.support.wait import WebDriverWait
-from selenium.webdriver.remote.webelement import WebElement as WE
+
+from qrunner import conf
 from qrunner.utils.log import logger
-from qrunner.utils.exceptions import ElementTypeError, NoSuchElementException
+from qrunner.utils.exceptions import ElementTypeError, NoSuchElementException, ElementNameEmptyException
 from qrunner.core.web.driver import WebDriver
 
 
@@ -19,7 +19,7 @@ LOC_LIST = {
     'name': By.NAME,
     'link_text': By.LINK_TEXT,
     'tag_name': By.TAG_NAME,
-    'partial_link_text': By.PARTIAL_LINK_TEXT,
+    'p_link_text': By.PARTIAL_LINK_TEXT,
     'class_name': By.CLASS_NAME,
     'xpath': By.XPATH,
     'css': By.CSS_SELECTOR
@@ -28,26 +28,69 @@ LOC_LIST = {
 
 class WebElement:
     """
-    Web元素定义
+    根据定位方式定位元素并进行操作
     """
-    def __init__(self, driver, **kwargs):
-        self._index = kwargs.pop('index', 0)
-        self.desc = kwargs.pop('desc', '')
+    def __init__(self,
+                 id_=None,
+                 name=None,
+                 link_text=None,
+                 p_link_text=None,
+                 tag_name=None,
+                 class_name=None,
+                 xpath=None,
+                 css=None,
+                 index=None,
+                 desc=None):
+        """
+        param tid: 标签的id
+        param name: 标签的name属性,
+        param ltext: a标签的文本，精准匹配,
+        param pltext: a标签的文本，模糊匹配,
+        param tname: 标签名,
+        param cname: 标签的class属性,
+        param xpath,
+        param css,
+        param index: 索引,
+        param desc: 控件名称
+        """
+        # 参数处理
+        kwargs = {}
+        if id_ is not None:
+            kwargs['id_'] = id_
+        if name is not None:
+            kwargs['name'] = name
+        if link_text is not None:
+            kwargs['link_text'] = link_text
+        if p_link_text is not None:
+            kwargs['p_link_text'] = p_link_text
+        if tag_name is not None:
+            kwargs['tag_name'] = tag_name
+        if class_name is not None:
+            kwargs['class_name'] = class_name
+        if xpath is not None:
+            kwargs['xpath'] = xpath
+        if css is not None:
+            kwargs['css'] = css
+        self._index = index if index is not None else 0
+        self.desc = desc
+        if self.desc is None:
+            raise ElementNameEmptyException('请设置控件名称')
 
-        if not kwargs:
-            raise ElementTypeError('请输入定位方式')
-
-        if len(kwargs) > 1:
-            raise ElementTypeError('请仅指定一种定位方式')
+        # if not kwargs:
+        #     raise ElementTypeError('请输入定位方式')
+        #
+        # if len(kwargs) > 1:
+        #     raise ElementTypeError('请仅指定一种定位方式')
 
         self.k, self.v = next(iter(kwargs.items()))
-        # print(self.k, self.v)
-        if self.k not in LOC_LIST.keys():
-            raise ElementTypeError(f'不支持的定位方式: {self.k}，仅支持: {LOC_LIST.keys()}')
-
+        # # print(self.k, self.v)
+        # if self.k not in LOC_LIST.keys():
+        #     raise ElementTypeError(f'不支持的定位方式: {self.k}，仅支持: {LOC_LIST.keys()}')
         self._kwargs = kwargs
-        self._element = None
-        self.driver: WebDriver = driver
+
+        # 驱动初始化
+        browser_name = conf.get_item('web', 'browser_name')
+        self.driver = WebDriver.get_instance(browser_name)
         self.d = self.driver.d
 
     def _wait(self, timeout=3):
@@ -59,11 +102,11 @@ class WebElement:
             return False
 
     def _find_element(self, retry=3, timeout=5):
+        cur_retry = retry
         while not self._wait(timeout=timeout):
-            if retry > 0:
-                retry -= 1
-                logger.info(f'重试 查找元素 {self._kwargs}')
-                time.sleep(2)
+            if cur_retry > 0:
+                logger.warning(f'第{retry-cur_retry+1}次重试，查找元素： {self._kwargs}')
+                cur_retry -= 1
             else:
                 frame = inspect.currentframe().f_back
                 caller = inspect.getframeinfo(frame)
@@ -72,46 +115,33 @@ class WebElement:
         elements = self.d.find_elements(LOC_LIST[self.k], self.v)
         return elements
 
-    def show_element(self, elem):
-        """
-                Show the elements of the operation
-                :param elem:
-                """
-        style_red = 'arguments[0].style.border="2px solid #FF0000"'
-        style_blue = 'arguments[0].style.border="2px solid #00FF00"'
-        style_null = 'arguments[0].style.border=""'
-        for _ in range(2):
-            self.driver.execute_js(style_red, elem)
-            time.sleep(0.1)
-            self.driver.execute_js(style_blue, elem)
-            time.sleep(0.1)
-        self.driver.execute_js(style_blue, elem)
-        time.sleep(0.3)
-        self.driver.execute_js(style_null, elem)
-
     def get_elements(self, retry=3, timeout=3):
         elements = self._find_element(retry=retry, timeout=timeout)
         if elements is None:
             self.driver.screenshot(f'[控件 {self.desc} 定位失败]')
             raise NoSuchElementException(f'[控件 {self.desc} 定位失败]')
         else:
-            element: WE = elements[self._index]
-            self.show_element(element)
-            # self.driver.screenshot(f'{loc}-定位成功')
+            for element in elements:
+                self.driver.execute_js("arguments[0].style.border='3px solid red'", element)
+            self.driver.screenshot(self.desc)
         return elements
 
+    # def __getitem__(self, index):
+    #     elements = self.get_elements()
+    #     return elements[index]
+
     def get_element(self, retry=3, timeout=3):
-        elements = self._find_element(retry=retry, timeout=timeout)
-        if elements is None:
-            self.driver.screenshot(f'[控件 {self.desc} 定位失败]')
-            raise NoSuchElementException(f'[控件 {self.desc} 定位失败]')
-        else:
-            element: WE = elements[self._index]
+        # elements = self._find_element(retry=retry, timeout=timeout)
+        # if elements is None:
+        #     self.driver.screenshot(f'[控件 {self.desc} 定位失败]')
+        #     raise NoSuchElementException(f'[控件 {self.desc} 定位失败]')
         # else:
-        #     element: WE = elements[self._index]
-        #     self.show_element(element)
-            # self.driver.screenshot(f'[{self.desc}]-[定位成功]')
-        return element
+        #     element = elements[self._index]
+        #     self.driver.execute_js("arguments[0].style.border='3px solid red'", element)
+        #     self.driver.screenshot(self.desc)
+        # return element
+        elements = self.get_elements(retry=retry, timeout=timeout)
+        return elements[self._index]
 
     def exists(self, timeout=1):
         logger.info(f'判断元素: {self._kwargs} 是否存在')
@@ -121,13 +151,13 @@ class WebElement:
             return False
         return True
 
-    def click(self):
+    def click(self, retry=3, timeout=5):
         logger.info(f'点击元素: {self._kwargs}')
-        self.get_element().click()
+        self.get_element(retry=retry, timeout=timeout).click()
 
-    def click_exists(self):
+    def click_exists(self, timeout=1):
         logger.info(f'存在才点击元素: {self._kwargs},{self._index}')
-        if self.exists():
+        if self.exists(timeout=timeout):
             self.click()
 
     def slow_click(self):
@@ -256,13 +286,6 @@ class WebElement:
         logger.info(text)
         return text
 
-    def get_texts(self):
-        logger.info(f'获取元素 {self._kwargs} 文本列表')
-        elements = self.get_elements()
-        texts = [elem.text for elem in elements]
-        logger.info(texts)
-        return texts
-
     def select_index(self, index):
         logger.info(f'选择第 {index} 个下拉列表')
         element = self.get_element()
