@@ -6,7 +6,7 @@ from email.header import Header
 
 
 # 邮件通知
-from qrunner.utils.allure_data import AllureData
+from qrunner.utils.allure_data import get_allure_data
 from qrunner.utils.config import conf
 
 
@@ -52,8 +52,7 @@ class Mail:
         conn.quit()
 
     def send_report(self, title, report_url, receiver_list):
-        report_path = conf.get_item('common', 'report_path')
-        allure_data = AllureData(report_path).get_basic_info()
+        allure_data = get_allure_data('allure-results')
         total = allure_data.get('total')
         fail = allure_data.get('fail')
         passed = allure_data.get('passed')
@@ -76,3 +75,4 @@ class Mail:
 
 
 
+
